import { AuthAPI } from "../api-definition";
import { v4 as uuidv4 } from "uuid";
import { UserRegister, UserUpdate, ShopLoginAccessTokenData } from "@/client";
import { extractSubFromJWT, generateFakeJWT } from "@/utils/jwt";
import { mockDB } from "./db";
import { MockUser, Shop } from "./models";

const db = mockDB;

export const MockAuthAPI: AuthAPI = {
	async getCurrentUser() {
		const token = localStorage.getItem("access_token");
		if (!token) return null;

		const userUUID = extractSubFromJWT(token);
		if (!userUUID) return null;

		const user = await db.users.where("uuid").equals(userUUID).first();
		if (!user) return null;

		const publicUser = {
			...user,
			first_name: user.first_name ?? null,
			last_name: user.last_name ?? null,
			profile_picture: user.profile_picture ?? null,
		};
		return publicUser;
	},

	async registerUser(data: UserRegister) {
		const now = new Date().toISOString();
		const userId = Date.now();
		const userUUID = uuidv4();
		const shopUUID = uuidv4();

		const newUser: MockUser = {
			id: userId,
			uuid: userUUID,
			user_role: "owner",
			status: "owner",
			shop_id: null,
			username: data.username,
			email: data.email,
			password: data.password,
			first_name: undefined,
			last_name: undefined,
			phone_number: data.phone_number,
			profile_picture: undefined,
			created_at: now,
			updated_at: now,
			last_login: now,
		};

		const newShop: Shop = {
			id: userId,
			uuid: shopUUID,
			owner_id: userId,
			name: `${data.username}'s Shop`,
			description: "A newly created shop.",
			created_at: now,
			updated_at: now,
			status: "inactive",
			logo: null,
			contact_email: data.email,
			contact_phone_number: data.phone_number,
			currency: "USD",
			address: {
				street: "",
				city: "",
				state: null,
				postal_code: "",
				country: "",
			},
		};

		await mockDB.transaction(
			"rw",
			mockDB.users,
			mockDB.shops,
			mockDB.preferences,
			mockDB.statistics,
			async () => {
				await mockDB.users.add(newUser);
				await mockDB.preferences.add({ user_id: userId });
				await mockDB.statistics.add({ user_id: userId, total_spend: 0 });
				await mockDB.shops.add(newShop);
			},
		);
	},

	async loginUser(data: ShopLoginAccessTokenData) {
		const user = await db.users
			.where("email")
			.equals(data.formData.username)
			.first();

		if (!user || user.password != data.formData.password) {
			throw new Error("Invalid credentials");
		}

		await db.users.update(user.id, { last_login: new Date().toISOString() });

		const token = generateFakeJWT(user.uuid);
		localStorage.setItem("access_token", token);
		return { access_token: token };
	},

	async updateUser(data: UserUpdate) {
		const token = localStorage.getItem("access_token");
		if (!token) return;

		const userUUID = extractSubFromJWT(token);
		if (!userUUID) return;

		await db.users
			.where("uuid")
			.equals(userUUID)
			.modify({
				updated_at: new Date().toISOString(),
				email: data.email ?? "",
				phone_number: data.phone_number ?? "",
				username: data.username ?? "",
				first_name: data.first_name ?? undefined,
				last_name: data.last_name ?? undefined,
			});
	},
};
