import { mockDB } from "./db";
import { ProductCategory, ProductCreate, ProductWithDetails } from "./models";
import { getCurrentUserDirect } from "./utils/currentUser";

export const MockProductAPI = {
	async getProductsForShop(): Promise<ProductWithDetails[]> {
		const user = await getCurrentUserDirect();
		if (!user?.id) return [];

		const products = await mockDB.products
			.where("shop_id")
			.equals(user.id)
			.toArray();

		const detailedProducts = await Promise.all(
			products.map(async (product) => {
				const images = await mockDB.product_images
					.where("product_id")
					.equals(product.id)
					.toArray();
				const variants = await mockDB.product_variants
					.where("product_id")
					.equals(product.id)
					.toArray();
				const categoryLinks = await mockDB.product_category_junctions
					.where("product_id")
					.equals(product.id)
					.toArray();
				const rawCategories = await Promise.all(
					categoryLinks.map((link) =>
						mockDB.product_categories.get(link.category_id),
					),
				);
				const categories = rawCategories.filter(
					(c): c is ProductCategory => c !== undefined,
				);

				return {
					...product,
					images,
					variants,
					categories,
				};
			}),
		);

		return detailedProducts;
	},

	async getProductById(productId: number): Promise<ProductWithDetails | null> {
		const product = await mockDB.products.get(productId);
		if (!product) return null;

		const images = await mockDB.product_images
			.where("product_id")
			.equals(productId)
			.toArray();
		const categoryLinks = await mockDB.product_category_junctions
			.where("product_id")
			.equals(productId)
			.toArray();

		const rawCategories = await Promise.all(
			categoryLinks.map((link) =>
				mockDB.product_categories.get(link.category_id),
			),
		);
		const categories = rawCategories.filter(
			(c): c is ProductCategory => c !== undefined,
		);

		return {
			...product,
			images,
			categories,
		};
	},

	async createProduct(productData: ProductCreate) {
		const user = await getCurrentUserDirect();
		if (!user?.id) return null;

		const now = new Date().toISOString();
		const productId = Date.now();

		await mockDB.products.add({
			id: productId,
			...productData,
			shop_id: user.id,
			created_at: now,
			updated_at: now,
		});

		if ("image_data" in productData && productData.image_data) {
			const image_id = Date.now();
			await mockDB.product_images.add({
				id: image_id,
				product_id: productId,
				image_id: image_id,
				image_url: productData.image_data,
				alt_text: `${productData.name} image`,
			});
		}

		return mockDB.products.get(productId);
	},

	async updateProduct(productId: number, data: Partial<ProductCreate>) {
		const product = await mockDB.products.get(productId);
		if (!product) return null;

		const updatedProduct = {
			...product,
			...data,
			updated_at: new Date().toISOString(),
		};
		await mockDB.products.put(updatedProduct);

		if (data.image_data) {
			const existingImage = await mockDB.product_images
				.where("product_id")
				.equals(productId)
				.first();
			if (existingImage) {
				await mockDB.product_images.put({
					...existingImage,
					image_url: data.image_data,
					alt_text: `${data.name ?? product.name} image`,
				});
			} else {
				const image_id = Date.now();
				await mockDB.product_images.add({
					id: image_id,
					product_id: productId,
					image_id: image_id,
					image_url: data.image_data,
					alt_text: `${data.name ?? product.name} image`,
				});
			}
		}

		return updatedProduct;
	},

	async deleteProduct(productId: number) {
		await mockDB.product_images.where("product_id").equals(productId).delete();
		await mockDB.product_variants
			.where("product_id")
			.equals(productId)
			.delete();
		await mockDB.product_category_junctions
			.where("product_id")
			.equals(productId)
			.delete();
		await mockDB.products.delete(productId);
		return true;
	},
};
