import { mockDB } from "./db";
import {
	Purchase,
	PurchaseWithDetails,
	PurchaseEntry,
	PurchaseEntryWithProduct,
} from "./models";

export const MockPurchaseAPI = {
	async getAllPurchases(): Promise<Purchase[]> {
		return mockDB.purchases.toArray();
	},

	async getPurchaseById(purchaseId: number): Promise<Purchase | null> {
		return (await mockDB.purchases.get(purchaseId)) ?? null;
	},

	async getPurchaseWithDetails(
		purchaseId: number,
	): Promise<PurchaseWithDetails | null> {
		const purchase = await mockDB.purchases.get(purchaseId);
		if (!purchase) return null;

		const [user, coupon, entries] = await Promise.all([
			mockDB.users.get(purchase.user_id),
			purchase.used_coupon_id
				? mockDB.coupons.get(purchase.used_coupon_id)
				: null,
			mockDB.purchase_entries.where("purchase_id").equals(purchaseId).toArray(),
		]);

		if (!user) return null;

		const entriesWithProducts: PurchaseEntryWithProduct[] = await Promise.all(
			entries.map(async (entry) => {
				const product = await mockDB.products.get(entry.product_id);
				if (!product) throw new Error("Product not found");
				return { ...entry, product };
			}),
		);

		return {
			...purchase,
			user,
			coupon,
			entries: entriesWithProducts,
		};
	},

	async getPurchasesByUser(userId: number): Promise<Purchase[]> {
		return mockDB.purchases.where("user_id").equals(userId).toArray();
	},

	async createPurchase(
		purchase: Omit<Purchase, "id">,
		entries: Omit<PurchaseEntry, "id">[] = [],
	): Promise<Purchase> {
		const id = Date.now();
		const newPurchase: Purchase = {
			...purchase,
			id,
		};

		await mockDB.purchases.add(newPurchase);

		if (entries.length > 0) {
			await mockDB.purchase_entries.bulkAdd(
				entries.map((entry) => ({
					...entry,
					purchase_id: id,
					id: Date.now() + Math.random(),
				})),
			);
		}

		return newPurchase;
	},

	async updatePurchase(
		purchaseId: number,
		updates: Partial<Omit<Purchase, "id">>,
	): Promise<Purchase | null> {
		const existing = await mockDB.purchases.get(purchaseId);
		if (!existing) return null;

		const updated = { ...existing, ...updates };
		await mockDB.purchases.put(updated);
		return updated;
	},

	async deletePurchase(purchaseId: number): Promise<boolean> {
		await mockDB.purchase_entries
			.where("purchase_id")
			.equals(purchaseId)
			.delete();
		await mockDB.purchases.delete(purchaseId);
		return true;
	},
};
