import { Button } from "@/components/ui/button";
import { DialogFooter } from "@/components/ui/dialog";
import { Input } from "@/components/ui/input";
import { Label } from "@/components/ui/label";
import { UseFormReturn } from "react-hook-form";
import { z } from "zod";

// eslint-disable-next-line @typescript-eslint/no-unused-vars
const schema = z.object({
	name: z.string().min(1, "Name is required"),
	description: z.string().min(1, "Description is required"),
	price: z.coerce.number().min(0),
	stock_quantity: z.coerce.number().int().min(0),
	image_data: z.string().optional(),
});

type ProductForm = z.infer<typeof schema>;

export function ProductForm({
	onSubmit,
	form,
	onImageUpload,
	submitLabel,
}: {
	onSubmit: () => void;
	form: UseFormReturn<ProductForm>;
	imagePreview?: string;
	onImageUpload: (e: React.ChangeEvent<HTMLInputElement>) => void;
	submitLabel: string;
}) {
	return (
		<form onSubmit={onSubmit} className="grid gap-4 py-4">
			<div className="grid gap-1">
				<Label htmlFor="name">Product Name</Label>
				<Input {...form.register("name")} placeholder="Product name" />
				{form.formState.errors.name && (
					<p className="text-sm text-red-500">
						{form.formState.errors.name.message}
					</p>
				)}
			</div>
			<div className="grid gap-1">
				<Label htmlFor="description">Description</Label>
				<Input
					{...form.register("description")}
					placeholder="Short description"
				/>
				{form.formState.errors.description && (
					<p className="text-sm text-red-500">
						{form.formState.errors.description.message}
					</p>
				)}
			</div>
			<div className="grid grid-cols-2 gap-4">
				<div className="grid gap-1">
					<Label htmlFor="price">Price</Label>
					<Input
						type="number"
						{...form.register("price", { valueAsNumber: true })}
						placeholder="0.00"
					/>
				</div>
				<div className="grid gap-1">
					<Label htmlFor="stock">Stock</Label>
					<Input
						type="number"
						{...form.register("stock_quantity", { valueAsNumber: true })}
						placeholder="0"
					/>
				</div>
			</div>
			<div className="grid gap-1">
				<Label htmlFor="image">Image</Label>
				<Input
					id="image"
					type="file"
					accept="image/jpg,image/png"
					onChange={onImageUpload}
				/>
				{form.watch("image_data") && (
					<img
						src={form.watch("image_data")}
						alt="Preview"
						className="mt-2 h-24 w-24 rounded border object-cover"
					/>
				)}
			</div>
			<DialogFooter>
				<Button type="submit">{submitLabel}</Button>
			</DialogFooter>
		</form>
	);
}
