import {
	IconAdjustmentsHorizontal,
	IconSortAscendingLetters,
	IconSortDescendingLetters,
	IconPlus,
} from "@tabler/icons-react";

import { Header } from "@/components/layout/header";
import { Main } from "@/components/layout/main";
import { Search } from "@/components/search";
import { ThemeSwitch } from "@/components/theme-switch";
import { ProfileDropdown } from "@/components/profile-dropdown";
import { Button } from "@/components/ui/button";
import { Input } from "@/components/ui/input";
import {
	Select,
	SelectTrigger,
	SelectValue,
	SelectContent,
	SelectItem,
} from "@/components/ui/select";
import { Separator } from "@/components/ui/separator";

import { useProducts } from "@/hooks/useProducts";
import { useProduct } from "@/hooks/useProduct";
import { useState } from "react";
import { useProductForm } from "./hooks/use-product-form";
import { ProductList } from "./components/product-list";
import { ProductDialog } from "./components/product-dialog";
import { toast } from "@/hooks/useToast";
import { useShop } from "@/hooks/useShop";

export default function Products() {
	const [sort, setSort] = useState<"ascending" | "descending">("ascending");
	const [searchTerm, setSearchTerm] = useState("");
	const [dialogOpen, setDialogOpen] = useState(false);
	const [editDialogOpen, setEditDialogOpen] = useState(false);
	const [selectedProductId, setSelectedProductId] = useState<
		number | undefined
	>(undefined);

	const { shop } = useShop();
	const { data: products = [], isLoading } = useProducts();
	const { product, createProduct, updateProduct, deleteProduct } =
		useProduct(selectedProductId);

	const form = useProductForm(product.data);

	const handleImageUpload = (e: React.ChangeEvent<HTMLInputElement>) => {
		const file = e.target.files?.[0];
		if (!file) return;
		const reader = new FileReader();
		reader.onload = () => form.setValue("image_data", reader.result as string);
		reader.readAsDataURL(file);
	};

	const handleCreateClicked = () => {
		form.reset({
			name: "",
			description: "",
			price: 0,
			stock_quantity: 0,
			image_data: "",
		});
		setSelectedProductId(undefined);
		setDialogOpen(true);
	};

	const handleCreateSubmit = form.handleSubmit((data) => {
		createProduct.mutate(data);
		setDialogOpen(false);
		toast({ title: `${data.name} created`, variant: "default" });
	});

	const handleEditSubmit = form.handleSubmit((data) => {
		if (!selectedProductId) return;
		updateProduct.mutate(data);
		setEditDialogOpen(false);
		toast({ title: `${data.name} saved`, variant: "default" });
	});

	const handleDelete = () => {
		if (!selectedProductId) return;
		deleteProduct.mutate();
		setEditDialogOpen(false);
		toast({ title: `${product.data?.name} deleted`, variant: "destructive" });
	};

	const sortedProducts = [...products].sort((a, b) =>
		sort === "ascending"
			? a.name.localeCompare(b.name)
			: b.name.localeCompare(a.name),
	);

	const filteredProducts = sortedProducts.filter((product) =>
		product.name.toLowerCase().includes(searchTerm.toLowerCase()),
	);

	return (
		<>
			<Header>
				<Search />
				<div className="ml-auto flex items-center gap-4">
					<ThemeSwitch />
					<ProfileDropdown />
				</div>
			</Header>

			<Main fixed>
				<div className="flex items-center justify-between">
					<div>
						<h1 className="text-2xl font-bold tracking-tight">Products</h1>
						<p className="text-muted-foreground">
							Here's a list of your products!
						</p>
					</div>
				</div>

				<div className="flex flex-col gap-4 sm:my-4 sm:flex-row">
					<Input
						placeholder="Search products..."
						className="h-9 w-40 lg:w-[250px]"
						value={searchTerm}
						onChange={(e) => setSearchTerm(e.target.value)}
					/>
					<Select
						value={sort}
						onValueChange={(v) => setSort(v as "ascending" | "descending")}
					>
						<SelectTrigger className="w-16">
							<SelectValue>
								<IconAdjustmentsHorizontal size={18} />
							</SelectValue>
						</SelectTrigger>
						<SelectContent align="end">
							<SelectItem value="ascending">
								<div className="flex items-center gap-4">
									<IconSortAscendingLetters size={16} />
									<span>Ascending</span>
								</div>
							</SelectItem>
							<SelectItem value="descending">
								<div className="flex items-center gap-4">
									<IconSortDescendingLetters size={16} />
									<span>Descending</span>
								</div>
							</SelectItem>
						</SelectContent>
					</Select>
				</div>

				<Separator className="shadow" />

				<ProductList
					currency={shop?.currency ?? ""}
					products={filteredProducts}
					isLoading={isLoading}
					onClick={(id) => {
						setSelectedProductId(id);
						setEditDialogOpen(true);
					}}
				/>

				<Button
					onClick={() => handleCreateClicked()}
					className="absolute bottom-0 z-10 mb-4 size-14 rounded-full p-0 shadow-lg hover:brightness-110"
					variant="default"
				>
					<IconPlus size={28} />
				</Button>
			</Main>

			{/* Create Dialog */}
			<ProductDialog
				open={dialogOpen}
				onOpenChange={setDialogOpen}
				form={form}
				onSubmit={handleCreateSubmit}
				onImageUpload={handleImageUpload}
				submitLabel="Create"
				imagePreview={form.watch("image_data")}
			/>

			{/* Edit Dialog */}
			<ProductDialog
				open={editDialogOpen}
				onOpenChange={setEditDialogOpen}
				form={form}
				onSubmit={handleEditSubmit}
				onImageUpload={handleImageUpload}
				onDelete={handleDelete}
				submitLabel="Update"
				imagePreview={form.watch("image_data")}
				showDeleteButton
			/>
		</>
	);
}
