"use client";

import { z } from "zod";
import { useForm } from "react-hook-form";
import { zodResolver } from "@hookform/resolvers/zod";
import { toast } from "@/hooks/useToast";
import { Button } from "@/components/ui/button";
import {
	Dialog,
	DialogContent,
	DialogDescription,
	DialogFooter,
	DialogHeader,
	DialogTitle,
} from "@/components/ui/dialog";
import {
	Form,
	FormControl,
	FormField,
	FormItem,
	FormLabel,
	FormMessage,
} from "@/components/ui/form";
import { Input } from "@/components/ui/input";
import { ScrollArea } from "@/components/ui/scroll-area";
import { Coupon } from "../data/schema";
import { useCoupon } from "@/hooks/useCoupon";

const formSchema = z.object({
	name: z.string().min(1, { message: "Name is required." }),
	text: z.string().min(1, { message: "Text is required." }),
	valid_due: z.coerce.date({
		errorMap: () => ({ message: "Invalid date." }),
	}),
	discount_amount: z
		.number({ invalid_type_error: "Must be a number." })
		.min(1, { message: "Discount must be at least 1%." })
		.max(100, { message: "Discount can't exceed 100%." }),
	isEdit: z.boolean(),
});

type CouponForm = z.infer<typeof formSchema>;

interface Props {
	currentRow?: Coupon;
	open: boolean;
	onOpenChange: (open: boolean) => void;
}

export function CouponsActionDialog({ currentRow, open, onOpenChange }: Props) {
	const { createCoupon, updateCoupon } = useCoupon(currentRow?.id);
	const isEdit = !!currentRow;

	const form = useForm<CouponForm>({
		resolver: zodResolver(formSchema),
		defaultValues: isEdit
			? {
					...currentRow,
					isEdit,
				}
			: {
					name: "",
					text: "",
					valid_due: new Date(),
					discount_amount: 10,
					isEdit,
				},
	});

	const onSubmit = (values: CouponForm) => {
		try {
			const payload = {
				...values,
				valid_due: values.valid_due.toISOString(), // convert to string
			};

			if (isEdit) {
				updateCoupon.mutate({ ...currentRow, ...payload });
			} else {
				createCoupon.mutate({
					...payload,
				});
			}

			toast({
				title: isEdit ? "Coupon updated" : "Coupon created",
				description: `${values.name} (${values.discount_amount}%)`,
			});
			form.reset();
			onOpenChange(false);
		} catch (err) {
			toast({
				title: "An error occurred",
				description: (err as Error).message,
				variant: "destructive",
			});
		}
	};

	return (
		<Dialog
			open={open}
			onOpenChange={(state) => {
				form.reset();
				onOpenChange(state);
			}}
		>
			<DialogContent className="sm:max-w-lg">
				<DialogHeader className="text-left">
					<DialogTitle>{isEdit ? "Edit Coupon" : "Add New Coupon"}</DialogTitle>
					<DialogDescription>
						{isEdit ? "Update the coupon below." : "Create a new coupon."}
					</DialogDescription>
				</DialogHeader>

				<ScrollArea className="-mr-4 h-[26.25rem] w-full py-1 pr-4">
					<Form {...form}>
						<form
							id="coupon-form"
							onSubmit={form.handleSubmit(onSubmit)}
							className="space-y-4 p-0.5"
						>
							<FormField
								control={form.control}
								name="name"
								render={({ field }) => (
									<FormItem>
										<FormLabel>Name</FormLabel>
										<FormControl>
											<Input placeholder="Spring Sale" {...field} />
										</FormControl>
										<FormMessage />
									</FormItem>
								)}
							/>
							<FormField
								control={form.control}
								name="text"
								render={({ field }) => (
									<FormItem>
										<FormLabel>Text</FormLabel>
										<FormControl>
											<Input placeholder="Get 20% off all items!" {...field} />
										</FormControl>
										<FormMessage />
									</FormItem>
								)}
							/>
							<FormField
								control={form.control}
								name="discount_amount"
								render={({ field }) => (
									<FormItem>
										<FormLabel>Discount (%)</FormLabel>
										<FormControl>
											<Input
												type="number"
												placeholder="e.g., 10"
												{...field}
												onChange={(e) => field.onChange(Number(e.target.value))}
											/>
										</FormControl>
										<FormMessage />
									</FormItem>
								)}
							/>
							<FormField
								control={form.control}
								name="valid_due"
								render={({ field }) => (
									<FormItem>
										<FormLabel>Valid Until</FormLabel>
										<FormControl>
											<Input
												type="datetime-local"
												value={field.value.toISOString().slice(0, 16)}
												onChange={(e) =>
													field.onChange(new Date(e.target.value))
												}
											/>
										</FormControl>
										<FormMessage />
									</FormItem>
								)}
							/>
						</form>
					</Form>
				</ScrollArea>

				<DialogFooter>
					<Button type="submit" form="coupon-form">
						Save changes
					</Button>
				</DialogFooter>
			</DialogContent>
		</Dialog>
	);
}
