import { ColumnDef } from "@tanstack/react-table";
import { Checkbox } from "@/components/ui/checkbox";
import { cn } from "@/lib/utils";
import { Coupon } from "../data/schema";
import { DataTableColumnHeader } from "./data-table-column-header";
import { DataTableRowActions } from "./data-table-row-actions";
import { Badge } from "@/components/ui/badge";
import { format, isBefore } from "date-fns";
import { couponStatusColors, couponTypes } from "../data/data";

export const columns: ColumnDef<Coupon>[] = [
	{
		id: "select",
		header: ({ table }) => (
			<Checkbox
				checked={
					table.getIsAllPageRowsSelected() ||
					(table.getIsSomePageRowsSelected() && "indeterminate")
				}
				onCheckedChange={(value) => table.toggleAllPageRowsSelected(!!value)}
				aria-label="Select all"
				className="translate-y-[2px]"
			/>
		),
		meta: {
			className: cn(
				"sticky md:table-cell left-0 z-10 rounded-tl",
				"bg-background transition-colors duration-200 group-hover/row:bg-muted group-data-[state=selected]/row:bg-muted",
			),
		},
		cell: ({ row }) => (
			<Checkbox
				checked={row.getIsSelected()}
				onCheckedChange={(value) => row.toggleSelected(!!value)}
				aria-label="Select row"
				className="translate-y-[2px]"
			/>
		),
		enableSorting: false,
		enableHiding: false,
	},
	{
		accessorKey: "name",
		header: ({ column }) => (
			<DataTableColumnHeader column={column} title="Name" />
		),
		cell: ({ row }) => (
			<div className="font-medium">{row.getValue("name")}</div>
		),
		enableHiding: false,
	},
	{
		accessorKey: "text",
		header: ({ column }) => (
			<DataTableColumnHeader column={column} title="Text" />
		),
		cell: ({ row }) => (
			<div className="max-w-64 truncate text-muted-foreground">
				{row.getValue("text")}
			</div>
		),
	},
	{
		accessorKey: "Valid Due",
		header: ({ column }) => (
			<DataTableColumnHeader column={column} title="Valid Until" />
		),
		cell: ({ row }) => {
			const { valid_due } = row.original;
			return <div>{format(new Date(valid_due), "PPPp")}</div>;
		},
	},
	{
		accessorKey: "status",
		header: ({ column }) => (
			<DataTableColumnHeader column={column} title="Status" />
		),
		cell: ({ row }) => {
			const validDue = new Date(row.original.valid_due);
			const now = new Date();

			const isExpired = isBefore(validDue, now);
			const status = isExpired ? "expired" : "active";
			const badgeColor = couponStatusColors.get(status);
			const statusMeta = couponTypes.find((t) => t.value === status);

			return (
				<div className="flex items-center gap-x-2">
					{statusMeta?.icon && (
						<statusMeta.icon size={16} className="text-muted-foreground" />
					)}
					<Badge variant="outline" className={cn("capitalize", badgeColor)}>
						{statusMeta?.label}
					</Badge>
				</div>
			);
		},
		filterFn: (row, id, value) => {
			console.log(id);
			const validDue = new Date(row.original.valid_due);
			const now = new Date();
			const status = isBefore(validDue, now) ? "expired" : "active";
			return value.includes(status);
		},
		enableSorting: false,
		enableHiding: false,
	},
	{
		accessorKey: "Discount",
		header: ({ column }) => (
			<DataTableColumnHeader column={column} title="Discount" />
		),
		cell: ({ row }) => {
			const { discount_amount } = row.original;
			return <Badge variant="outline">{discount_amount}%</Badge>;
		},
	},
	{
		id: "actions",
		cell: DataTableRowActions,
	},
];
