import { ColumnDef } from "@tanstack/react-table";
import { Checkbox } from "@/components/ui/checkbox";
import { cn } from "@/lib/utils";
import { Purchase } from "../data/schema";
import { DataTableColumnHeader } from "./data-table-column-header";
import { DataTableRowActions } from "./data-table-row-actions";
import { format } from "date-fns";
import { Badge } from "@/components/ui/badge";
import { Coupon } from "@/api/mock/models";

export function getColumns(
	couponsMap: Map<number, Coupon>
): ColumnDef<Purchase>[] {
	return [
		{
			id: "select",
			header: ({ table }) => (
				<Checkbox
					checked={
						table.getIsAllPageRowsSelected() ||
						(table.getIsSomePageRowsSelected() && "indeterminate")
					}
					onCheckedChange={(value) => table.toggleAllPageRowsSelected(!!value)}
					aria-label="Select all"
					className="translate-y-[2px]"
				/>
			),
			meta: {
				className: cn(
					"sticky md:table-cell left-0 z-10 rounded-tl",
					"bg-background transition-colors duration-200 group-hover/row:bg-muted group-data-[state=selected]/row:bg-muted"
				)
			},
			cell: ({ row }) => (
				<Checkbox
					checked={row.getIsSelected()}
					onCheckedChange={(value) => row.toggleSelected(!!value)}
					aria-label="Select row"
					className="translate-y-[2px]"
				/>
			),
			enableSorting: false,
			enableHiding: false
		},
		{
			accessorKey: "id",
			header: ({ column }) => (
				<DataTableColumnHeader column={column} title="Purchase ID" />
			),
			cell: ({ row }) => (
				<div className="font-medium">{row.getValue("id")}</div>
			),
			enableHiding: false
		},
		{
			accessorKey: "total",
			header: ({ column }) => (
				<DataTableColumnHeader column={column} title="Total" />
			),
			cell: ({ row }) => {
				const value = row.getValue("total") as number;
				return <div className="font-medium">{value.toFixed(2)}</div>;
			}
		},
		{
			accessorKey: "date_purchased",
			header: ({ column }) => (
				<DataTableColumnHeader column={column} title="Date" />
			),
			cell: ({ row }) => {
				const date = new Date(row.getValue("date_purchased"));
				return <div>{format(date, "PPPp")}</div>;
			}
		},
		{
			accessorKey: "used_coupon_id",
			header: ({ column }) => (
				<DataTableColumnHeader column={column} title="Coupon" />
			),
			cell: ({ row }) => {
				const { used_coupon_id } = row.original;
				const coupon = used_coupon_id ? couponsMap.get(used_coupon_id) : null;
				return coupon ? (
					<Badge variant="outline">{coupon.name}</Badge>
				) : (
					<span className="text-muted-foreground">—</span>
				);
			}
		},
		{
			id: "actions",
			cell: DataTableRowActions
		}
	];
}
