import { ColumnDef } from "@tanstack/react-table";
import { cn } from "@/lib/utils";
import { Checkbox } from "@/components/ui/checkbox";
import LongText from "@/components/long-text";
import { callTypes, userTypes } from "../data/data";
import { User } from "../data/schema";
import { DataTableColumnHeader } from "./data-table-column-header";
import { DataTableRowActions } from "./data-table-row-actions";
import { Badge } from "@/components/ui/badge";

export const columns: ColumnDef<User>[] = [
	{
		id: "select",
		header: ({ table }) => (
			<Checkbox
				checked={
					table.getIsAllPageRowsSelected() ||
					(table.getIsSomePageRowsSelected() && "indeterminate")
				}
				onCheckedChange={(value) => table.toggleAllPageRowsSelected(!!value)}
				aria-label="Select all"
				className="translate-y-[2px]"
			/>
		),
		meta: {
			className: cn(
				"sticky md:table-cell left-0 z-10 rounded-tl",
				"bg-background transition-colors duration-200 group-hover/row:bg-muted group-data-[state=selected]/row:bg-muted",
			),
		},
		cell: ({ row }) => (
			<Checkbox
				checked={row.getIsSelected()}
				onCheckedChange={(value) => row.toggleSelected(!!value)}
				aria-label="Select row"
				className="translate-y-[2px]"
			/>
		),
		enableSorting: false,
		enableHiding: false,
	},
	{
		accessorKey: "username",
		header: ({ column }) => (
			<DataTableColumnHeader column={column} title="Username" />
		),
		cell: ({ row }) => (
			<LongText className="max-w-36">{row.getValue("username")}</LongText>
		),
		meta: {
			className: cn(
				"drop-shadow-[0_1px_2px_rgb(0_0_0_/_0.1)] dark:drop-shadow-[0_1px_2px_rgb(255_255_255_/_0.1)] lg:drop-shadow-none",
				"bg-background transition-colors duration-200 group-hover/row:bg-muted group-data-[state=selected]/row:bg-muted",
				"sticky left-6 md:table-cell",
			),
		},
		enableHiding: false,
	},
	{
		accessorKey: "email",
		header: ({ column }) => (
			<DataTableColumnHeader column={column} title="Email" />
		),
		cell: ({ row }) => (
			<div className="w-fit text-nowrap">{row.getValue("email")}</div>
		),
	},
	{
		id: "fullName",
		header: ({ column }) => (
			<DataTableColumnHeader column={column} title="Name" />
		),
		cell: ({ row }) => {
			const { first_name: firstName, last_name: lastName } = row.original;

			if (firstName == undefined || lastName == undefined) {
				return (
					<LongText className="max-w-36 text-muted select-none">
						{"No name available"}
					</LongText>
				);
			}

			const fullName = `${firstName} ${lastName}`;
			return <LongText className="max-w-36">{fullName}</LongText>;
		},
		meta: { className: "w-36" },
	},
	{
		accessorKey: "Phone",
		header: ({ column }) => (
			<DataTableColumnHeader column={column} title="Phone Number" />
		),
		cell: ({ row }) => {
			const { phone_number } = row.original;
			return <div>{phone_number}</div>;
		},
		enableSorting: false,
	},
	{
		accessorKey: "role",
		header: ({ column }) => (
			<DataTableColumnHeader column={column} title="Role" />
		),
		cell: ({ row }) => {
			const { user_role: role } = row.original;
			const userType = userTypes.find(({ value }) => value === role);

			if (!userType) {
				return null;
			}
			const badgeColor = callTypes.get(role);

			return (
				<div className="flex items-center gap-x-2">
					{userType.icon && (
						<userType.icon size={16} className="text-muted-foreground" />
					)}
					<Badge variant="outline" className={cn("capitalize", badgeColor)}>
						{role}
					</Badge>
				</div>
			);
		},
		filterFn: (row, id, value) => {
			return value.includes(row.getValue(id));
		},
		enableSorting: false,
		enableHiding: false,
	},
	{
		id: "actions",
		cell: DataTableRowActions,
	},
];
